=== modified file 'lib/canonical/launchpad/components/externalbugtracker/trac.py'
--- lib/canonical/launchpad/components/externalbugtracker/trac.py	2008-09-12 09:39:36 +0000
+++ lib/canonical/launchpad/components/externalbugtracker/trac.py	2008-10-04 20:32:43 +0000
@@ -11,11 +11,13 @@
 import urllib2
 import xmlrpclib
 
+from cookielib import CookieJar
 from datetime import datetime
 from email.Utils import parseaddr
 from zope.component import getUtility
 from zope.interface import implements
 
+from canonical.cachedproperty import cachedproperty
 from canonical.config import config
 from canonical.launchpad.components.externalbugtracker.base import (
     BugNotFound, ExternalBugTracker, InvalidBugId, LookupTree,
@@ -286,12 +288,15 @@
         ISupportsBackLinking, ISupportsCommentImport, ISupportsCommentPushing)
 
     def __init__(self, baseurl, xmlrpc_transport=None,
-                 internal_xmlrpc_transport=None):
+                 internal_xmlrpc_transport=None, cookie_jar=None):
         super(TracLPPlugin, self).__init__(baseurl)
 
+        if cookie_jar is None:
+            cookie_jar = CookieJar()
         if xmlrpc_transport is None:
-            xmlrpc_transport = UrlLib2Transport(baseurl)
+            xmlrpc_transport = UrlLib2Transport(baseurl, cookie_jar)
 
+        self._cookie_jar = cookie_jar
         self._xmlrpc_transport = xmlrpc_transport
         self._internal_xmlrpc_transport = internal_xmlrpc_transport
 
@@ -299,6 +304,14 @@
         self._server = xmlrpclib.ServerProxy(
             xmlrpc_endpoint, transport=self._xmlrpc_transport)
 
+    @cachedproperty
+    def urlopener(self):
+        """Return a urllib2.OpenerDirector that handles cookies."""
+        opener = urllib2.build_opener(
+            urllib2.HTTPCookieProcessor(self._cookie_jar))
+
+        return opener
+
     @needs_authentication
     def initializeRemoteBugDB(self, bug_ids):
         """See `IExternalBugTracker`."""
@@ -312,6 +325,15 @@
             if remote_bug['status'] != 'missing':
                 self.bugs[int(remote_bug['id'])] = remote_bug
 
+    def urlopen(self, request, data=None):
+        """See `ExternalBugTracker`.
+
+        This method is overridden here so that it uses the urlopener
+        property in order to maintain the use of Trac authentication
+        cookies across requests.
+        """
+        return self.urlopener.open(request, data)
+
     def _generateAuthenticationToken(self):
         """Create an authentication token and return it."""
         internal_xmlrpc = xmlrpclib.ServerProxy(
@@ -333,8 +355,6 @@
         base_auth_url = urlappend(self.baseurl, 'launchpad-auth')
         auth_url = urlappend(base_auth_url, token_text)
         response = self.urlopen(auth_url)
-        auth_cookie = self._extractAuthCookie(response.headers['Set-Cookie'])
-        self._xmlrpc_transport.setCookie(auth_cookie)
 
     @needs_authentication
     def getCurrentDBTime(self):

=== modified file 'lib/canonical/launchpad/doc/externalbugtracker-trac-lp-plugin.txt'
--- lib/canonical/launchpad/doc/externalbugtracker-trac-lp-plugin.txt	2008-09-12 09:39:36 +0000
+++ lib/canonical/launchpad/doc/externalbugtracker-trac-lp-plugin.txt	2008-10-04 20:32:43 +0000
@@ -54,10 +54,15 @@
     ...             raise AssertionError("Token has already been consumed.")
     ...         token.consume()
     ...         print "Successfully validated the token."
+    ...
+    ...         cookie_string = (
+    ...             'trac_auth=random_token-' + str(random.random()))
+    ...         self._xmlrpc_transport.setCookie(cookie_string)
+    ...
     ...         response = FakeResponse()
-    ...         response.headers['Set-Cookie'] = (
-    ...             'trac_auth=random_token-' + str(random.random()))
+    ...         response.headers['Set-Cookie'] = cookie_string
     ...         LaunchpadZopelessLayer.switchDbUser(config.checkwatches.dbuser)
+    ...
     ...         return response
 
 To generate the token, the internal XML-RPC server is used. By using the
@@ -65,12 +70,16 @@
 don't have to bother about commiting the transaction to make the token
 visible to Trac.
 
+    >>> from cookielib import CookieJar
     >>> from canonical.launchpad.ftests.externalbugtracker import (
     ...     TestInternalXMLRPCTransport)
-    >>> test_transport = TestTracXMLRPCTransport('http://example.com/')
+    >>> cookie_jar = CookieJar()
+    >>> test_transport = TestTracXMLRPCTransport(
+    ...     'http://example.com/', cookie_jar)
     >>> trac = TestTracLPPlugin(
     ...     'http://example.com/', xmlrpc_transport=test_transport,
-    ...     internal_xmlrpc_transport=TestInternalXMLRPCTransport())
+    ...     internal_xmlrpc_transport=TestInternalXMLRPCTransport(),
+    ...     cookie_jar=cookie_jar)
 
 The method that authenticates with Trac is _authenticate().
 
@@ -84,25 +93,14 @@
     >>> test_transport.cookie_processor.cookiejar
     <cookielib.CookieJar[Cookie(version=0, name='trac_auth'...
 
-There's another method, _extractAuthCookie(), which gets the cookie from
-the Set-Cookie header.
-
-    >>> trac._extractAuthCookie(
-    ...     'trac_auth=some_token; expires=Some date; path=/some/path;'
-    ...     ' domain=some.name.example.com; secure')
-    'trac_auth=some_token'
-
-    >>> print trac._extractAuthCookie(
-    ...     'other_cookie=some_token; expires=Some date; path=/some/path;'
-    ...     ' domain=some.name.example.com; secure')
-    None
-
-The XML-RPC transport will then take the auth_cookie and set the Cookie
-header in its requests.
-
-    >>> trac = TracLPPlugin('http://example.com/')
-    >>> transport = trac._xmlrpc_transport
-    >>> transport.auth_cookie = 'trac_auth=random_token'
+The XML-RPC transport's cookie_processor shares its cookiejar with the
+TracLPPlugin instance. This is so that the TracLPPlugin can use the
+cookiejar when authenticating with the remote Trac and then pass it to
+the XML-RPC transport for further use, meaning that there's no need to
+manually manipulate cookies.
+
+    >>> test_transport.cookie_processor.cookiejar == trac._cookie_jar
+    True
 
 
 == Current time ==

=== modified file 'lib/canonical/launchpad/ftests/externalbugtracker-xmlrpc-transport.txt'
--- lib/canonical/launchpad/ftests/externalbugtracker-xmlrpc-transport.txt	2008-10-04 19:55:55 +0000
+++ lib/canonical/launchpad/ftests/externalbugtracker-xmlrpc-transport.txt	2008-10-04 20:32:43 +0000
@@ -28,7 +28,7 @@
     >>> from cookielib import CookieJar
     >>> jar = CookieJar()
     >>> transport = UrlLib2Transport('http://example.com', jar)
-    >>> transport.cookie_processor.cookiejar is jar
+    >>> transport.cookie_processor.cookiejar == jar
     True
 
 We patch the opener to return a fixed response without actually opening

